/* -*- pse-c -*-
 *----------------------------------------------------------------------------
 * Filename: intel_escape.h
 * $Revision: 1.30 $
 *----------------------------------------------------------------------------
 *
 * Copyright (c) Intel Corporation (2004 - 2005).
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors.  Title to the Material
 * remains with Intel Corporation or it suppliers and licensors.  The
 * Material contains trade secrets and proprietary and confidential
 * information of Intel or its suppliers and licensors.  The Material is
 * protected by worldwide copyright and trade secret laws and
 * treaty provisions.  No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way without Intels prior express
 * written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license
 * under such intellectual property rights must be express
 * and approved by Intel in writing.
 *
 * File Description:
 *   Include file for Intel's driver control protocol and client library
 *
 *----------------------------------------------------------------------------
 * Functions:
 *
 *----------------------------------------------------------------------------
 */
/* $XFree86: $ */
/* $Xorg: Exp $ */

/* THIS IS NOT AN X CONSORTIUM STANDARD OR AN X PROJECT TEAM SPECIFICATION */

#ifndef _INTEL_ESCAPE_H_
#define _INTEL_ESCAPE_H_

#include <X11/Xfuncproto.h>
#include <X11/Xmd.h>

#define X_IegdQueryVersion           0
#define X_IegdSetClientVersion       1
#define X_IegdEscape                 2

#define MAX_NAME_LEN 256

#define INTEL_ESCAPE_NOT_SUPPORTED  -1
#define INTEL_ESCAPE_SUCCESS         0
#define INTEL_ESCAPE_ERROR           1

#define INTEL_ESCAPE_NO_REPLY                0x01000 /* don't send reply */

#define INTEL_ESCAPE_SET_MODE                0x20000
#define INTEL_ESCAPE_GET_NUM_MODES           0x20001
#define INTEL_ESCAPE_GET_AVAIL_MODES         0x20002
#define INTEL_ESCAPE_QUERY_NUM_AVAIL_PORTS   0x20003 /* deprecated */
#define INTEL_ESCAPE_QUERY_AVAIL_PORTS       0x20004 /* deprecated */
#define INTEL_ESCAPE_ENABLE_PORT             0x20005
#define INTEL_ESCAPE_MULTI_DISPLAY           0x20006 /* deprecated */
#define INTEL_ESCAPE_GET_NUM_PD_ATTRIBUTES   0x20007
#define INTEL_ESCAPE_GET_AVAIL_PD_ATTRIBUTES 0x20008
#define INTEL_ESCAPE_SET_PD_ATTRIBUTES       0x20009
#define INTEL_ESCAPE_I2C_CONFIG              0x2000A /* Not support on X */
#define INTEL_ESCAPE_I2C_ACCESS              0x2000B /* Not support on X */
#define INTEL_ESCAPE_GET_NUM_DISPLAYS        0x2000C /* deprecated */
#define INTEL_ESCAPE_ENUM_DISPLAYS           0x2000D /* deprecated */
#define INTEL_ESCAPE_SET_ROTATION_FLIP       0x2000E
#define INTEL_ESCAPE_GET_ROTATION_FLIP       0x2000F
#define INTEL_ESCAPE_SET_FB_GAMMA_RAMP	     0x20011 /* deprecated */
#define INTEL_ESCAPE_GET_DRIVER_INFO         0x20012
#define INTEL_ESCAPE_GET_CONFIG_INFO         0x20013 /* deprecated */
#define INTEL_ESCAPE_GET_PORT_INFO           0x20014
#define INTEL_ESCAPE_GET_CURRENT_DC          0x20015
#define INTEL_ESCAPE_GET_DC_LIST             0x20016
#define INTEL_ESCAPE_SET_DC                  0x20017
#define INTEL_ESCAPE_GET_NUM_DC              0x20018
#define INTEL_ESCAPE_GET_OVL_COLOR_PARAMS    0x20019
#define INTEL_ESCAPE_SET_OVL_COLOR_PARAMS    0x2001A 
#define INTEL_ESCAPE_NOOP                    0x20020
#define INTEL_ESCAPE_NOOP_DRAWABLE          (0x20021 | INTEL_ESCAPE_NO_REPLY)
#define INTEL_ESCAPE_GET_EDID_INFO           0x20024 /* Num same as Windows */

/* X driver specific commands */
#define INTEL_ESCAPE_PUT_SURFACE             0x20100

/* Direct Rendering Related Commands */
#define INTEL_ESCAPE_BUFFER_INFO             0x20201
#define INTEL_ESCAPE_ALLOC_SURFACE           0x20202
#define INTEL_ESCAPE_FREE                   (0x20203 | INTEL_ESCAPE_NO_REPLY)
#define INTEL_ESCAPE_SYNC                    0x20204
#define INTEL_ESCAPE_ALLOC_REGION            0x20205
#define INTEL_ESCAPE_FLIP                   (0x20206 | INTEL_ESCAPE_NO_REPLY)
#define INTEL_ESCAPE_EXEC                    0x20207
#define INTEL_ESCAPE_FILL                    0x20208
#define INTEL_ESCAPE_BLIT                    0x20209

/*
 * The following typedefs define the input and output structures
 * that are passed between the driver and the client.  These are
 * versioned, and must not be changed without incrementing the
 * protocal version number.
 */

typedef struct _iegd_esc_status {
	int status;
} iegd_esc_status_t;

/* Port number input */
typedef struct _iegd_esc_port_in {
	unsigned long port_number;
} iegd_esc_port_in_t;

/* DC value input/output.  On output an array of this type may be returned */
typedef struct _iegd_esc_dc {
	unsigned long dc;
} iegd_esc_dc_t;

/* For the functions that return a count it will return this */
typedef struct _iegd_esc_count {
	unsigned long count;
} iegd_esc_count_t;


/* Port attributes */
typedef struct _iegd_esc_attr {
	unsigned long id;
	unsigned long type;
	char name[32];
	unsigned long flags;
	unsigned long default_value;
	unsigned long current_value;
	unsigned long _pad0;
	unsigned long _pad1;
	unsigned long _pad2;
} iegd_esc_attr_t;

/* Input to driver when setting a display mode */
typedef struct _iegd_esc_mode {
	unsigned long width;
	unsigned long height;
	unsigned long refresh;
	unsigned long depth;
	unsigned long port;  /* needed to independently set clone's mode */
} iegd_esc_mode_t;

/* Input to driver when querying mode table */
typedef struct _iegd_esc_mode_in {
	unsigned long port_number;
	unsigned long dc;
} iegd_esc_mode_in_t;

/* Input to driver to enable/disable a port */
typedef struct _iegd_esc_port_ctrl {
	int port;
	int enable;
} iegd_esc_port_ctrl_t;

/* Input to driver with a new attribute list */
typedef struct _iegd_esc_set_attr {
	unsigned long port_number;
	iegd_esc_attr_t attribute;
} iegd_esc_set_attr_t;


/* Input to driver with new DC, timings, and framebuffer config */
typedef struct _iegd_esc_set_dc {
	unsigned long dc;
	struct _timing {
		unsigned short width;
		unsigned short height;
		unsigned short refresh;
	} iegd_timings[2];
	struct _fb {
		unsigned short width;
		unsigned short height;
		unsigned short bit_depth;
	} iegd_fb_info[2];
} iegd_esc_set_dc_t;


/* Input to driver with surface information */
typedef struct _iegd_esc_surface_info {
	unsigned long screen;           /* Screen number */
	unsigned long surface_id;       /* Surface to put */
	unsigned long drawable_id;      /* drawable */
	unsigned short src_x;           /* source X */
	unsigned short src_y;           /* source Y */
	unsigned short src_w;           /* source width */
	unsigned short src_h;           /* source height */
	unsigned short dst_x;           /* destination X */
	unsigned short dst_y;           /* destination Y */
	unsigned short dst_w;           /* destination width */
	unsigned short dst_h;           /* destination height */
	unsigned long flags;            /* surface flags */
} iegd_esc_surface_info_t;

/* Input to driver with drawable info */
typedef struct _iegd_esc_drawable_info {
	unsigned long drawable_id;
} iegd_esc_drawable_info_t;

/* Input to driver with rotation/flip information */
typedef struct _iegd_esc_set_rotation_flip {
	unsigned long port_number;
	unsigned long rotation; /* rotation in degrees (0, 90, 180, 270) */
	unsigned long flip;     /* horizontal flip, zero or non-zero */
} iegd_esc_set_rotation_flip_t;

/* Output from driver with rotation/flip information */
typedef struct _iegd_esc_get_rotation_flip {
	unsigned long rotation; /* rotation in degrees (0, 90, 180, 270) */
	unsigned long flip;     /* horizontal flip, zero or non-zero */
} iegd_esc_get_rotation_flip_t;

/* Output from driver */
typedef struct _iegd_esc_driver_info {
	char name[MAX_NAME_LEN];       /* IEGD Driver Name                */
	char chipset[MAX_NAME_LEN];    /* Chipset name                    */
	unsigned long major;            /* Major version see igd_version.h */
	unsigned long minor;            /* Minor version see igd_version.h */
	unsigned long build;            /* Build number see igd_version.h  */
	char date[MAX_NAME_LEN];       /* Date of build                   */
} iegd_esc_driver_info_t;

/* Output from driver with information about all ports */
typedef struct _iegd_esc_port_info {
	unsigned long width;            /* Current mode: screen width     */
	unsigned long height;           /* Current mode: screen height    */
	unsigned long depth;            /* Current mode: screen depth     */
	unsigned long refresh;          /* Current mode: screen refresh   */
	unsigned long edid;             /* Edid enable, 0 or 1            */

	unsigned long enable;           /* Port is enabled or disabled    */
	unsigned long timing_owner;     /* Set if port owns timings       */

	char pd_name[MAX_NAME_LEN];     /* Port driver name see pd.h      */
	char user_alias[MAX_NAME_LEN];  /* User(via IAL reg/cfg file)port name */
	unsigned long pd_version;       /* Port driver version see pd.h   */
	unsigned long pd_type;          /* Port driver type see pd.h      */
} iegd_esc_port_info_t;

/* Output from driver with current mode list */
typedef struct _iegd_esc_mode_list {
	unsigned short width;
	unsigned short height;
	unsigned short refresh;
	unsigned long dclk;              /* in KHz */
	unsigned short h_sync_start;
	unsigned short h_sync_length;
	unsigned short h_blank_start;
	unsigned short h_blank_length;
	unsigned short v_sync_start;
	unsigned short v_sync_length;
	unsigned short v_blank_start;
	unsigned short v_blank_length;
	unsigned short mode_number;
	unsigned long flags;
} iegd_esc_mode_list_t;

/* EDID info returned */
typedef struct _iegd_esc_edid_info {
	unsigned char edid[128];
} iegd_esc_edid_info_t;


#define GAMMA_FLAG			0x1
#define BRIGHTNESS_FLAG		0x2
#define CONTRAST_FLAG		0x4
#define SATURATION_FLAG		0x8
#define OVL_COLOR_FLAG		0x10

/*
 * the following escape structure is for dynamically changing the overlay
 * color attributes or frame buffer gamma correction
 *  ** this backdoor is temporary and is for MPD only! **
 */
typedef struct _iegd_esc_color_params {
	unsigned char flag; /* 0x1 = gamma, 0x2 = brightness */
						/* 0x4 = contrast, 0x8 = saturation */
						/*0x10 = Overlay colors; if not set for FB */
	unsigned int gamma;
	unsigned int brightness;
	unsigned int contrast;
	unsigned int saturation;
} iegd_esc_color_params_t;

typedef struct _iegd_esc_i2c_config {
	char bus;  /* I2C bus */
	char dab;  /* 7 bit device address byte */
	unsigned long speed; /* i2c bus speed */
	unsigned long enable; /* Internal use only */
} iegd_esc_i2c_config_t;

#define INTEL_I2C_MODE_READ  1
#define INTEL_I2C_MODE_WRITE 2

typedef struct _iegd_esc_i2c_access {
	char addr; /* Register offset */
	char data; /* Data to write (or read from device) */
	unsigned long mode; /* read or write */
} iegd_esc_i2c_access_t;

#define MULTI_TYPE_SINGLE      0x1
#define MULTI_TYPE_CLONE       0x2
#define MULTI_TYPE_TWIN        0x4
#define MULTI_TYPE_EXTENDED    0x8

/*
 * The buffer info escape details the front buffer information for
 * the requested screen. It also returns the sync slot used by that screen.
 * This escape is used with direct clients but can be called before the
 * direct connection is established.
 */
typedef struct _iegd_esc_buffer_info {
	unsigned int screen;
} iegd_esc_buffer_info_t;

typedef struct _iegd_esc_buffer_info_reply {
	unsigned int width;
	unsigned int height;
	unsigned int pitch;
	unsigned int pixel_format;
	unsigned int flags;
	unsigned int offset;
	unsigned int sync_slot;
} iegd_esc_buffer_info_reply_t;

/*
 * Structures needed for surface allocation and free from the internal
 * memory manager.
 */
typedef struct _iegd_esc_alloc_surface {
	unsigned int drm_context;
	unsigned int surface_type;
	unsigned int width;
	unsigned int height;
	unsigned int pixel_format;
	unsigned int flags;
} iegd_esc_alloc_surface_t;

/*
 * For Normal surfaces only offset[0] is returned.
 * For MIPMAP surfaces up to 12 offsets are returned based on the number
 *  of available levels of detail.
 * For Cubemap surfaces the Positive left surface offset is returned.
 * For Cubemap + Mipmap the offsets for each level of detail for the
 *  positive left face are returned. The client can then calculate the
 *  offsets for all LOD of all faces.
 */
typedef struct _iegd_esc_alloc_surface_reply {
	int ret;
	unsigned int buffer_id;
	unsigned int width;
	unsigned int height;
	unsigned int pitch;
	unsigned int flags;
	unsigned int offset[12];
} iegd_esc_alloc_surface_reply_t;

/*
 * When allocating a volume surface the reply contains more offsets.
 */
typedef struct _iegd_esc_alloc_volume_surface_reply {
	int ret;
	unsigned int buffer_id;
	unsigned int width;
	unsigned int height;
	unsigned int pitch;
	unsigned int flags;
	unsigned int offset[511];
} iegd_esc_alloc_volume_surface_reply_t;

typedef struct _iegd_esc_alloc_region {
	unsigned int drm_context;
	unsigned int region_type;
	unsigned int size;
	unsigned int flags;
} iegd_esc_alloc_region_t;

typedef struct _iegd_esc_alloc_region_reply {
	int ret;
	unsigned int buffer_id;
	unsigned int offset;
	unsigned int size;
} iegd_esc_alloc_region_reply_t;

typedef struct _iegd_esc_free {
	unsigned int drm_context;
	unsigned int offset;
} iegd_esc_free_t;

typedef struct _iegd_esc_sync {
	unsigned int drm_context;
	unsigned int sync_id;
	unsigned int flags;
} iegd_esc_sync_t;

typedef struct _iegd_esc_sync_reply {
	unsigned int sync_id;
} iegd_esc_sync_reply_t;

typedef struct _iegd_esc_flip {
	unsigned int drm_context;
	unsigned int buffer_id;
} iegd_esc_flip_t;

/*
 * Drawable is needed for exec to make sure that the correct
 * ring buffers and syncs are used. These may be different on a
 * per-display basis.
 */
typedef struct _iegd_esc_exec {
	unsigned int drm_context;
	unsigned int drawable;
	unsigned int size;
	/* data */
} iegd_esc_exec_t;

typedef struct _iegd_esc_exec_reply {
	unsigned int sync_id;
} iegd_esc_exec_reply_t;

typedef struct _iegd_esc_fill {
	unsigned int drm_context;
	unsigned int buffer_id;
	unsigned int x1;
	unsigned int y1;
	unsigned int x2;
	unsigned int y2;
	unsigned int color;
} iegd_esc_fill_t;

typedef struct _iegd_esc_blit {
	unsigned int drm_context;
	unsigned int src_buffer_id;
	unsigned int dest_buffer_id;
	unsigned int drawable;
	unsigned int color;
	unsigned int src_x1;
	unsigned int src_y1;
	unsigned int src_x2;
	unsigned int src_y2;
	unsigned int dest_x1;
	unsigned int dest_y1;
	unsigned int dest_x2;
	unsigned int dest_y2;
} iegd_esc_blit_t;

#ifndef _IEGD_SERVER_

_XFUNCPROTOBEGIN

Bool iegd_query_version(
	Display*	/* dpy */,
	int*		/* majorVersion */,
	int*		/* minorVersion */
);

Bool iegd_query_extension(
	Display*	/* dpy */,
	int*		/* event_base */,
	int*		/* error_base */
);

Bool iegd_set_client_version(
	Display*	/* dpy */
);

int iegd_escape(
	Display*         /* dpy */,
	unsigned long    /* escape function code */,
	unsigned long    /* input data size */,
	void*            /* input data pointer */,
	unsigned long    /* output data size */,
	void*            /* output data pointer */
) ;

_XFUNCPROTOEND

#endif

#endif
